package examples.junit;

import de.uni_luebeck.isp.rvtool.junitrv.FLTL4Monitor;
import de.uni_luebeck.isp.rvtool.junitrv.Monitor;
import de.uni_luebeck.isp.rvtool.junitrv.Monitors;
import de.uni_luebeck.isp.rvtool.junitrv.RVRunner;
import de.uni_luebeck.isp.rvtool.javamonitorinjection.api.syntax.Event;
import static de.uni_luebeck.isp.rvtool.javamonitorinjection.api.syntax.SimpleSyntax.*;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;

/**
 * This is an example to demonstrate testing with jUnitRV.
 *
 * @author Normann Decker <decker@isp.uni-luebeck.de>
 *
 */
//Use jUnitRV to monitor test cases
@SuppressWarnings("serial")
@RunWith(RVRunner.class)
public class MyDataClient2Test {

    private static final String dataServiceQname
            = "examples/junit/DataService";
    private static final String dataClientQname
            = "examples/junit/";
    private static final Event modify = called(dataServiceQname,
            "modifyData");
    private static final Event committed = returned(dataServiceQname,
            "commit");
    private static final Event close = called(dataServiceQname,
            "disconnect");
    private static final Event opened = returned(dataServiceQname,
            "connect");
    // create a monitor for LTL4 property G(modify ⇒ ¬close U commit)
    private static Monitor commitBeforeClose = new FLTL4Monitor(
            Always(
                    implies(
                            modify,
                            Until(not(close), committed))));
    // create a monitor for LTL4 property
    //(¬modify W opened) ∧ G(close ⇒ ¬modify WU opened)
    private static Monitor openBeforeModify = new FLTL4Monitor(
            and(WU(not(modify), opened),
                    G(implies(close, WU(not(modify), opened)))));
    // ================= Eigenschaft 3 ================================
    private static final Event clientActionAuth
            = called(dataClientQname, "authenticate");
    private static final Event clientActionSwitch
            = called(dataClientQname, "switchToUser");
    private static final Event clientActionGetFile
            = called(dataClientQname, "getPatientFile");
    //...
    private static Monitor neverServiceUnset = new FLTL4Monitor(
            G(new WHEN(clientActionAuth, clientActionSwitch, clientActionGetFile) {
                public boolean p(MyDataClient client, String user) {
                    return client.getDataService() != null;
                }

                public boolean p(MyDataClient client, String user, String result) {
                    return client.getDataService() != null;
                }

            }));
 
    // ============================================================
    // ================ Eigenschaft 1 =============================
    private static final Event readData = called(dataServiceQname,
            "readData");
    private static final Event authenticated = called(dataClientQname,
            "authenticate");
    private static Monitor authBeforeAccess = new FLTL4Monitor(
            R(authenticated, not(or(modify, readData))));
    // ============================================================
    // ===================Eigenschaft 2 ===========================

    private static final Event setPhoneCall = called(dataClientQname, "setPhone");

    private static final Event setPhoneRet = returned(dataClientQname, "setPhone");

    private static final Predicate isphone = new WITH(modify) {
        public boolean p(DataService service, String field, String data) {
            return data.startsWith("phone:");
        }
    };

    private static final Monitor phone = new FLTL4Monitor(G(implies(setPhoneCall, U(not(setPhoneRet), isphone))));

    //eq(OBJECT(true),invoke("startsWith", $arg(1), OBJECT("") ))
    // ============================================================
    // ================ Eigenschaft 4 =============================
    private static final Event clientAddPatientCall
            = called(dataClientQname, "addPatient");
    private static final Event clientAddPatientRet
            = returned(dataClientQname, "addPatient");
    private static final Monitor modifyOnAddPatient = new FLTL4Monitor(
            G(implies(clientAddPatientCall, R(modify,
                                    not(clientAddPatientRet)))));

    // ============================================================
    @Ignore
    @Test
    @Monitors({"commitBeforeClose", "openBeforeModify",
        "authBeforeAccess"})
    public void test1() {
        DataService service = new MyDataService("http://myserver");
        MyDataClient client = new MyDataClient(service);
        client.authenticate("daniel");
        client.addPatient("Mr . Smith");
        client.switchToUser("ruth");
        client.getPatientFile("miller-2143-1");
        client.setPhone("miller-2143-1", "012345678");
        client.exit();
    }

    @Ignore
    @Test
    @Monitors({"commitBeforeClose", "openBeforeModify",
        "modifyOnAddPatient"})
    //@Monitors({"commitBeforeClose", "openBeforeModify"})
    public void test2() {
        DataService service = new MyDataService("http://myserver");
        MyDataClient client = new MyDataClient(service);
        client.authenticate("daniel");
        client.addPatient("Mr . Smith");
        client.addPatient("Mr . Smith");
    }

    @Test
    @Monitors({"phone"})
    public void test3() {
        DataService service = new MyDataService("http://myserver");
        MyDataClient client = new MyDataClient(service);
        client.setPhone("id", "123");
    }

    @Ignore
    @Test
    @Monitors({"authBeforeAccess"})
    public void testEig1() {
        DataService service = new MyDataService("http://myserver");
        MyDataClient client = new MyDataClient(service);
        //client.authenticate("daniel");
        client.getPatientFile("miller-2143-1");
        client.authenticate("daniel");
        client.getPatientFile("miller-2143-1");
    }
}
