package examples.junit;

import de.uni_luebeck.isp.rvtool.junitrv.FLTL4Monitor;
import de.uni_luebeck.isp.rvtool.junitrv.Monitor;
import de.uni_luebeck.isp.rvtool.junitrv.Monitors;
import de.uni_luebeck.isp.rvtool.junitrv.RVRunner;
import de.uni_luebeck.isp.rvtool.javamonitorinjection.api.syntax.Event;
import static de.uni_luebeck.isp.rvtool.javamonitorinjection.api.syntax.SimpleSyntax.*;
import de.uni_luebeck.isp.rvtool.salt.SaltMonitor;
import org.junit.Test;
import org.junit.runner.RunWith;

/**
 * This is an example to demonstrate testing with jUnitRV.
 *
 * @author Normann Decker <decker@isp.uni-luebeck.de>
 *
 */
//Use jUnitRV to monitor test cases
@RunWith(RVRunner.class)
@Monitors({"commitBeforeClose", "openBeforeModify"})
public class MyDataClientTest3 {

    private static final String dataServiceQname = "examples/junit/DataService";
    private static final Event modify = called(dataServiceQname, "modifyData");
    private static final Event committed = returned(dataServiceQname, "commit");
    private static final Event close = called(dataServiceQname, "disconnect");
    private static final Event opened = returned(dataServiceQname, "connect");
    
    // create a monitor for LTL4 property G(modify ⇒ ¬close U commit)
    private static final Monitor commitBeforeClose = new SaltMonitor("always(modify implies (not close until committed))", null, "eq");
    
    // create a monitor for LTL4 property 
    //(¬modify W opened) ∧ G(close ⇒ ¬modify WU opened)
    private static final Monitor openBeforeModify = new SaltMonitor("(not modify until weak opened) and always(close implies (not modify until weak opened))", null, "eq");

    @Test
    public void test1() {
        DataService service = new MyDataService("http://myserver");
        MyDataClient client = new MyDataClient(service);
        client.authenticate("daniel");
        client.addPatient("Mr . Smith");
        client.switchToUser("ruth");
        client.getPatientFile("miller-2143-1");
        client.setPhone("miller-2143-1", "012345678");
        client.exit();
    }

    @Test
    public void test2() {
        DataService service = new MyDataService("http://myserver");
        MyDataClient client = new MyDataClient(service);
        client.authenticate("daniel");
        client.addPatient("Mr . Smith");
        client.addPatient("Mr . Smith");
    }

    @Test
    public void test3() {
        DataService service = new MyDataService("http://myserver");
        MyDataClient client = new MyDataClient(service);
    }
}
